import {
  Action,
  getModule,
  Module,
  Mutation,
  MutationAction,
  VuexModule
} from 'vuex-module-decorators';
import findIndex from 'lodash.findindex';
import { Measure, MeasureConfig, Measurement, MeasurementMeta } from '@/models';
import assign from 'lodash.assign';
import api from '@/services/pm-api-client';
import { logService } from '@/services/logging';

export interface MeasurementStore { [key: string]: Array<Measurement<MeasurementMeta>>; }
export interface SetMeasurementsParameters {
  measure: Measure<MeasureConfig>;
  measurements: Array<Measurement<MeasurementMeta>>;
}

export interface MeasureAndMeasurement {
  measure: Measure<MeasureConfig>;
  measurement: Measurement<MeasurementMeta>;
}

const logger = logService.getLogger('/store-modules/measurement');

@Module({ namespaced: true, name: 'measurement' })
export class MeasurementStoreModule extends VuexModule {
  public measurements: MeasurementStore = {};

  @Action({ rawError: true })
  public async fetchMeasurements(measure: Measure<MeasureConfig>) {
    logger.trace('Fetching measurements for measure ' + measure.id);
    const measurements = await api.getMeasurements(measure.slug); // assumption: always returns at least []
    this.context.commit('SET_MEASUREMENTS', { measure, measurements });
  }

  @Action({ rawError: true })
  public async createMeasurement(data: MeasureAndMeasurement) {
    logger.trace('Creating new measurement for ' + data.measure.slug);
    const newMeasurement = await api.createMeasurement(data.measure.slug, data.measurement);
    this.context.commit('SET_MEASUREMENT', { measure: data.measure, measurement: newMeasurement });
  }

  @Mutation
  public SET_MEASUREMENTS({ measure: measure, measurements: measurements }: SetMeasurementsParameters) {
    this.measurements = assign({}, this.measurements, { [measure.id]: measurements });
  }

  @Mutation
  public SET_MEASUREMENT({ measure: measure, measurement: measurement }: MeasureAndMeasurement) {
    const existing = this.measurements[measure.id] || [];
    const newMeasurements = existing.slice();

    const index = findIndex(existing, { id: measurement.id });
    if (index < 0) { newMeasurements.push(measurement); }
    else { newMeasurements[index] = measurement; }
    this.measurements = assign({}, this.measurements, { [measure.id]: newMeasurements });
  }

}
